#!/usr/bin/env python
#
# PkgTrackInst.PkgDbInfoForm - Gets user input about pkgdb base information
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: PkgDbInfoForm.py 476 2005-05-01 14:38:12Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
import tempfile, re
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx, echo_msg

PKGDB_TEMPLATE = """\
# pkg_trackinst is about to install and track a package.  Please
# provide some informations about the package below to note in pkgdb.
# And then, save and quit.
#
PortName=\t%(portname)s
PortVersion=\t%(portversion)s
Comment=\tA package created with pkg_trackinst
Origin=\t\ttrackinst/%(portname)s
Prefix=\t\t%(prefix)s
Description=
A package created with pkg_trackinst.
And more descriptions..

WWW: http://www.FreeBSD.org/
"""
PKGDB_VARS = (
    'PortName', 'PortVersion', 'Comment', 'Origin', 'Prefix', 'Description'
)
PKGDBINFO_CACHE = './trackinst_cfg'

def get_pkgdb_info():
    if os.path.exists(PKGDBINFO_CACHE):
        echo_msg('Loading cached config: %s' % PKGDBINFO_CACHE)
        template = open(PKGDBINFO_CACHE).read()
        skip_editor = True
    else:
        vers = guess_versioninfo()
        vers['prefix'] = options.prefix

        template = PKGDB_TEMPLATE % vers
        skip_editor = False

    while True:
        if skip_editor or not options.interactive:
            raw = template
        else:
            raw = call_editor(template)

        try:
            parsed = parse_pkgdb_form(raw)
        except ValueError, exc:
            echo_msg(exc.args[0])

            decision = ' '
            while options.interactive and decision[:1].lower() not in 'ac':
                decision = raw_input('(a)gain or (c)ancel? ')

            if decision[0].lower() != 'a':
                raise SystemExit
            template = raw
            skip_editor = False
        else:
            break

    open(PKGDBINFO_CACHE, 'w').write(raw)
    return parsed

def guess_versioninfo():
    basename = os.path.basename(os.getcwd())
    if '-' not in basename:
        return {
            'portname': basename,
            'portversion': '1.0',
        }
    else:
        nfo = basename.rsplit('-', 1)
        return {
            'portname': nfo[0],
            'portversion': nfo[1],
        }

def call_editor(template):
    editor = os.environ.get('EDITOR', '/usr/bin/vi')

    tf = tempfile.NamedTemporaryFile(suffix='.sh')
    tf.write(template)
    tf.flush()

    mtm = os.path.getmtime(tf.name)
    status = os.system("'%s' '%s'" % (editor, tf.name))
    if os.path.getmtime(tf.name) == mtm:
        errx('canceled')
    if status != 0:
        errx('%s exited with status code %d' % (editor, status))
    return open(tf.name).read()

pat_begin_assignment = re.compile('([A-Z][A-Za-z]*)=\s*(.*)')
def parse_pkgdb_form(data):
    r = {}
    for varnam in PKGDB_VARS:
        r[varnam] = ''

    in_multiline = None
    for line in data.splitlines():
        found = pat_begin_assignment.match(line)

        if in_multiline:
            if line.startswith('#') or found:
                in_multiline = None
            else:
                r[in_multiline] += line + '\n'
                continue

        if not found:
            if line.startswith('#') or not line.strip():
                continue
            else:
                raise ValueError, 'invalid syntax around "%s"' % line[:15]

        label, value = found.groups()
        if label not in r:
            raise ValueError, "`%s' is unknown option name" % label
        if r[label]:
            raise ValueError, "`%s' is double assigned" % label

        if not value:
            in_multiline = label
        else:
            r[label] = value.strip()

    return r
