#!/usr/bin/env python
#
# PkgTrackInst.genplist_main - Tracks installation log and generates pkg-plist.
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: genplist_main.py 479 2005-05-02 14:09:01Z perky $

__pychecker__ = 'no-argsused no-isliteral'

from PkgTrackInst.cmd_support import *
import os, sys
import tempfile

options = None

def parse_options():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-s', '--no-substitute', dest='substitute',
                      default=True, action='store_false',
                      help="don't try any substitutions")
    opts, args = parser.parse_args()
    if args:
        parser.print_help()
        raise SystemExit
    return opts

def run_install():
    echo_msg('Building port')
    status = os.system('make build')
    if status:
        errx("`make build' is failed with status code %d." % status)

    echo_msg('Installing port')
    ilog = tempfile.NamedTemporaryFile()
    envs = 'LD_PRELOAD=%s INSTALLWATCHFILE=%s' % (INSTALLWATCH_PATH, ilog.name)
    status = os.system(envs + ' make install')
    if status:
        errx("`make install' is failed with status code %d." % status)
    return open(ilog.name)

def parse_log(logf):
    from PkgTrackInst.InstallSession import InstallSession
    inst = InstallSession()
    for line in logf:
        inst.feed(line[:-1])

def main():
    global options

    import PkgTrackInst
    PkgTrackInst.options = options = parse_options()

    # Prepare writer (to exit on error early)
    from PkgTrackInst.PlistWriter import PlistWriter
    writer = PlistWriter()

    # Run install command with libinstallwatch preloaded.
    logf = run_install()

    # Parse install log and build directory architecture.
    parse_log(logf)

    # Delete known temporary/build directories from the track list.
    from PkgTrackInst.Trackers import TrackRoot
    for dpfx in IGNORE_PATHS + [os.path.realpath(os.getcwd())]:
        del TrackRoot[dpfx]

    have_entry = []
    def found_entry(ent):
        if ent.created:
            have_entry.append(True)
    TrackRoot.walk(found_entry)
    if not have_entry:
        errx('No file is installed.')

    # Mark mtree directories as not created by the problem.
    from PkgTrackInst.Trackers import MTreeMarker
    MTreeMarker(writer.prefix, MTREE_LOCAL).mark()

    # Generate pkg-plist
    writer.write()

if __name__ == '__main__':
    main()
